@@ -2,4 +2,5 @@
 from .data_frame_parallel import DataFrameParallel
 from .series_parallel import SeriesParallel
 from .pandas_parallel_apply import apply_on_df_parallel, apply_on_df_maybe_parallel, \
-    apply_on_df_col_parallel, apply_on_df_col_maybe_parallel, apply_on_series_parallel, apply_on_series_maybe_parallel
\ No newline at end of file
+    apply_on_df_col_parallel, apply_on_df_col_maybe_parallel, apply_on_series_parallel, \
+    apply_on_series_maybe_parallel, apply_on_groupby_parallel
