@@ -1,10 +1,12 @@
 """General module for parallelizing a dataframe apply function on a column (series) or entire row"""
 
 import logging
+from tokenize import group
 from tqdm import tqdm
 from multiprocessing import Pool
 import numpy as np
 import pandas as pd
+from pandas.core.groupby.generic import DataFrameGroupBy
 from typing import Callable
 from functools import partial
 
@@ -22,6 +24,12 @@ def parallelize_dataframe(df: pd.DataFrame, func: Callable, n_cores: int) -> pd.
     pool.join()
     return df
 
+def parallelize_groupby(df_grouped: DataFrameGroupBy, func: Callable, n_cores: int) -> pd.DataFrame:
+    with Pool(n_cores) as p:
+        ret_list = p.map(func, [group for _, group in df_grouped])
+    return pd.concat(ret_list)
+
+
 """
 Apply on series
 Usage:
@@ -107,3 +115,30 @@ def apply_on_df_maybe_parallel(df: pd.DataFrame, f: Callable, parallel: bool,
         return apply_on_df_parallel(df, f, n_cores, pbar)
     else:
         return apply_on_df(df, f, pbar)
+
+"""
+Apply on df.groupby
+Usage:
+    standard => df.groupby([cols]).apply(f)
+    serial => apply_on_groupby_parallel(df.groubpy([cols]), f, n_cores)
+"""
+
+def _groupby_serial_func(data, func):
+    return [func(x) for x in tqdm(data)]
+
+def apply_on_groupby_parallel(df_grouped: DataFrameGroupBy, func: Callable, n_cores: int):
+    assert isinstance(df_grouped, DataFrameGroupBy)
+    grouped_data = []
+
+    # groupby returns a tuple (key, sub_df), we only care about the sub_df. Other use-cases may appear later.
+    for item in iter(df_grouped):
+        grouped_data.append(item[1])
+
+    n_cores = min(len(grouped_data), n_cores)
+    split_data = np.array_split(grouped_data, n_cores)
+    pool = Pool(n_cores)
+
+    res = pool.map(partial(_groupby_serial_func, func=func), split_data)
+    res = np.concatenate(res, dtype=object)
+    res = pd.DataFrame(res)
+    return res
